#!/usr/bin/perl

############################################################
# Hanzi counter v1.0 (Mar-05-2000)                         #
############################################################
# Copyright (C) 1999-2000 Kan-chan <kan-chan@innocent.com> #
#  Web site: http://kan-chan.stbbs.net/download/           #
############################################################
# This program is free software; you can redistribute it   #
# and/or modify it under the terms of the GNU General      #
# Public License as published by the Free Software         #
# Foundation; either version 2 of the License, or (at your #
# option) any later version.                               #
#                                                          #
# This program is distributed in the hope that it will be  #
# useful, but WITHOUT ANY WARRANTY; without even the       #
# implied warranty of MERCHANTABILITY or FITNESS FOR A     #
# PARTICULAR PURPOSE. See the GNU General Public License   #
# for more details.                                        #
#                                                          #
# You should have received a copy of the GNU General       #
# Public License along with this program; if not, write to #
# the Free Software Foundation, Inc., 675 Mass Ave,        #
# Cambridge, MA 02139, USA.                                #
############################################################

# Release note:
# v1.0 (Mar-05-2000)
#   First release.

# Usage:
# 1. Modify the perl directory written on the top of this script 
#    if it is required.
# 2. Modify the value of $basedir/$imagedir/$countfile/$lockfile etc. 
#    if the directory and/or file name are different.
# 3. Upload files to the following directories and change the 
#    permission.  If you cannot use 707/705/606, try 777/755/666.
# 4. Add <!--#exec cmd="./hzcount/count.pl"--> to index.html
#
# public_html/
#      |
#      |-- index.html
#      |-- .htaccess  (most systems require this to run SSI scripts)
#      |
#      |-- images/
#      |      |-- 0.gif (ling / zero)
#      |      |-- 1.gif (yi / ichi)
#      |      |-- 2.gif (er / ni)
#      |      |-- 3.gif (san / san)
#      |      |-- 4.gif (si / shi)
#      |      |-- 5.gif (wu / go)
#      |      |-- 6.gif (liu / roku)
#      |      |-- 7.gif (qi / nana)
#      |      |-- 8.gif (ba / hachi)
#      |      |-- 9.gif (jiu / kyu)
#      |      |-- s.gif (shi / ju)
#      |      |-- b.gif (bai / hyaku)
#      |      |-- q.gif (qian / sen)
#      |      |-- w.gif (wan / man)
#      |      |-- y.gif (yi / oku)
#      |      |-- ge.gif (ba) - for chinese mode
#      |      |-- ren.gif (ren / nin) - for chinese/japanese mode
#      |
#      |-- count/           (permission:707 or 777)
#             |-- hzcount.pl  (permission:705 or 755)
#             |-- hzcount.txt (permission:606 or 666)
#


############################################################
# Settings you may need to modify if necessary             #
############################################################
#
# Where you installed hzcount.pl/hzcount.txt (location from index.html)
$basedir = "./";
# Where you installed 0.gif~9.gif (location from index.html)
$imagedir = "./images/";
# The extension of the digit file (.gif / .jpg / .png  etc.)
$ext = ".gif";

# Japanese mode / Chinese mode
# There are differences between spellings in both languages.
# (Example: 103 = hyaku 3 in Japanese, 1 bai 0 3 in Chinese)

$mode = "JP";
#$mode = "CH";

# Direction (Horizontal / Vertical)
$direction = "H";
#$direction = "V";

# The name of count file
$countfile = "hzcount1.txt";
# The name of temporary file to lock the process
$lockfile = "hzcount1.loc";

# Error messages
$err_busy = "[Server busy!]";
$err_open = "[Cannot open count file! ($countfilepath)]";

############################################################

$countfilepath = $basedir.$countfile;
$lockfilepath = $basedir.$lockfile;

# Wait until the lock is released
for ($i=1;$i<=6;$i++){
	if (!(-e $lockfilepath)) {
		last;
	} elsif ($i == 1){
		$filetime = (stat($lockfilepath))[9];
		if ($filetime < time() - 10 * 60){
			&unlock();
		}
	} elsif ($i >= 6){
		&error($err_busy);
	}
	sleep(1);
}

# Create a lock file
open(OUT, ">$lockfilepath");
close(OUT);

# Read/write count file
if (!open(IN, "<$countfilepath")) {
	&unlock();
	&error($err_open);
}
$count = <IN>;
close(IN);
$count++;
if (!open(OUT, ">$countfilepath")) {
	&unlock();
	&error($err_open);
}
print(OUT $count);
close(OUT);

# Output a number as HTML tags
&disp($count);

# Remove the lock file
&unlock();

# Exit this program
exit;

# If a lock file exists, delete the file
sub unlock {
	if (-e $lockfilepath) { unlink($lockfilepath); }
}

# number - Japanese / Chinese
# 1 - 1 / 1
# 

# Output a number as HTML tags
sub disp {
	$count = $_[0];
	$count_length = length($count);
	for ($i = 12;$i>$count_length;$i--) { $count = " $count"; };

	$d = disp2(substr($count, 8, 4), '');
	if ($count_length >4) { $d = disp2(substr($count, 4, 4), 'w').$d; }
	if ($count_length >8) { $d = disp2(substr($count, 0, 4), 'y').$d; }
	$d =~ s/0+/0/g;
	if ($d eq '') { $d = '0'; }
	for ($i=0;$i<length($d);$i++){
		$n = substr($d, $i, 1);
		print "<img src=\"$imagedir$n$ext\" border=0>";
		if ($direction eq "V") { print "<br>\n"; }
	}
	if ($mode ne 'JP'){
		print "<img src=\"$imagedir"."ge"."$ext\" border=0>";
		if ($direction eq "V") { print "<br>\n"; }
	}
	print "<img src=\"$imagedir"."ren"."$ext\" border=0>";
	if ($direction eq "V") { print "<br>\n"; }
}

sub disp2 {
	$num = $_[0];
	$unit = $_[1];

	$dd = '';
	$num1 = substr($num, 3, 1);
	$num2 = substr($num, 2, 1);
	$num3 = substr($num, 1, 1);
	$num4 = substr($num, 0, 1);
	if ($mode eq 'JP') {
		if ($num4 eq '1') { $dd .= 'q'; }
		if ($num4 =~ '[2-9]') { $dd .= $num4.'q'; }
		if ($num3 eq '1') { $dd .= 'b'; }
		if ($num3 =~ '[2-9]') { $dd .= $num3.'b'; }
		if ($num2 eq '1') { $dd .= 's'; }
		if ($num2 =~ '[2-9]') { $dd .= $num2.'s'; }
		if ($num1 =~ '[1-9]') { $dd .= $num1; }
	} else{
		if ($num4 eq '0') { $dd .= '0'; }
		if ($num4 =~ '[1-9]') { $dd .= $num4.'q'; }
		if ($num3 eq '0') { $dd .= '0'; }
		if ($num3 =~ '[1-9]') { $dd .= $num3.'b'; }
		if ($num2 eq '0') { $dd .= '0'; }
		if ($num2 eq '1') { $dd .= 's'; }
		if ($num2 =~ '[2-9]') { $dd .= $num2.'s'; }
		if ($num1 =~ '[1-9]') { $dd .= $num1; }
		$dd =~ s/0+/0/g;
		$dd =~ s/0+$//g;
	}
	if ($dd ne ''){ $dd .= $unit; }
	return $dd;
}

# Output an error mesasge
sub error {
	print "$_[0]";
	exit;
}
